@@ -1,13 +1,28 @@
 #!/usr/bin/env python
 
+import sys
+import os.path
+
+def error(message, *args):
+    if args:
+        message = message % args
+    sys.stderr.write('ERROR: %s\n' % message)
+
 try:
     import lxml.etree as et
-except ImportError, e:
-    import sys
-    print >> sys.stderr, "ERR: %s." % e
+except ImportError:
+    error(sys.exc_info()[1])
     sys.exit(5)
 
-import sys, os.path, optparse, itertools
+try:
+    basestring
+except NameError:
+    basestring = (str, bytes)
+
+try:
+    unicode
+except NameError:
+    unicode = str
 
 SHORT_DESCRIPTION = "An XPath file finder for XML files."
 
@@ -79,15 +94,34 @@ being lexicographically within the interval ``a <= x <= b``.
 REGEXP_NS = "http://exslt.org/regular-expressions"
 PYTHON_BUILTINS_NS = "PYTHON-BUILTINS"
 
-parser = et.XMLParser(remove_blank_text=True)
+def make_parser(remove_blank_text=True, **kwargs):
+    return et.XMLParser(remove_blank_text=remove_blank_text, **kwargs)
 
-def print_result(result, pretty_print):
+def print_result(result, pretty_print, encoding=None, _is_py3=sys.version_info[0] >= 3):
+    stdout = sys.stdout
+    if not stdout.isatty() and not encoding:
+        encoding = 'utf8'
     if et.iselement(result):
-        result = et.tostring(result, xml_declaration=False,
-                             pretty_print=pretty_print)
-        if pretty_print:
-            result = result[:-1] # strip newline at the end
-    print result
+        result = et.tostring(result, xml_declaration=False, with_tail=False,
+                             pretty_print=pretty_print, encoding=encoding)
+        if not pretty_print:
+            # pretty printing appends newline, otherwise we do it
+            if isinstance(result, unicode):
+                result += '\n'
+            else:
+                result += '\n'.encode('ascii')
+    elif isinstance(result, basestring):
+        result += '\n'
+    else:
+        result = '%r\n' % result # '%r' for better number formatting
+
+    if encoding and encoding != 'unicode' and isinstance(result, unicode):
+        result = result.encode(encoding)
+
+    if _is_py3 and not isinstance(result, unicode):
+        stdout.buffer.write(result)
+    else:
+        stdout.write(result)
 
 def print_results(results, pretty_print):
     if isinstance(results, list):
@@ -96,57 +130,76 @@ def print_results(results, pretty_print):
     else:
         print_result(results, pretty_print)
 
-def find_in_file(f, xpath, print_name=True, xinclude=False, pretty_print=True):
-    if hasattr(f, 'name'):
-        filename = f.name
+def iter_input(input, filename, parser, line_by_line):
+    if isinstance(input, basestring):
+        with open(input, 'rb') as f:
+            for tree in iter_input(f, filename, parser, line_by_line):
+                yield tree
     else:
+        try:
+            if line_by_line:
+                for line in input:
+                    if line:
+                        yield et.ElementTree(et.fromstring(line, parser))
+            else:
+                yield et.parse(input, parser)
+        except IOError:
+            e = sys.exc_info()[1]
+            error("parsing %r failed: %s: %s",
+                  filename, e.__class__.__name__, e)
+
+def find_in_file(f, xpath, print_name=True, xinclude=False, pretty_print=True, line_by_line=False,
+                 encoding=None, verbose=True):
+    try:
+        filename = f.name
+    except AttributeError:
         filename = f
 
+    xml_parser = et.XMLParser(encoding=encoding)
+
     try:
-        try:
-            tree = et.parse(f, parser)
-        except IOError, e:
-            print >> sys.stderr, "ERR: parsing %r failed: %s: %s" % (
-                filename, e.__class__.__name__, e)
-            return False
-
-        try:
-            if xinclude:
-                tree.xinclude()
-        except IOError, e:
-            print >> sys.stderr, "ERR: XInclude for %r failed: %s: %s" % (
-                filename, e.__class__.__name__, e)
-            return False
-
         if not callable(xpath):
             xpath = et.XPath(xpath)
 
-        results = xpath(tree)
-        if results == []:
+        found = False
+        for tree in iter_input(f, filename, xml_parser, line_by_line):
+            try:
+                if xinclude:
+                    tree.xinclude()
+            except IOError:
+                e = sys.exc_info()[1]
+                error("XInclude for %r failed: %s: %s",
+                      filename, e.__class__.__name__, e)
+
+            results = xpath(tree)
+            if results is not None and results != []:
+                found = True
+                if verbose:
+                    print_results(results, pretty_print)
+
+        if not found:
             return False
-        if print_name:
-            print ">> %s" % f
-        if options.verbose:
-            print_results(results, pretty_print)
+        if not verbose and print_name:
+            print(filename)
         return True
-    except Exception, e:
-        print >> sys.stderr, "ERR: %r: %s: %s" % (
-            filename, e.__class__.__name__, e)
+    except Exception:
+        e = sys.exc_info()[1]
+        error("%r: %s: %s",
+              filename, e.__class__.__name__, e)
         return False
 
 def register_builtins():
     ns = et.FunctionNamespace(PYTHON_BUILTINS_NS)
     tostring = et.tostring
 
-    str_xpath = et.XPath("string()")
     def make_string(s):
         if isinstance(s, list):
             if not s:
-                return u''
+                return ''
             s = s[0]
         if not isinstance(s, unicode):
             if et.iselement(s):
-                s = tostring(s, method="text", encoding=unicode)
+                s = tostring(s, method="text", encoding='unicode')
             else:
                 s = unicode(s)
         return s
@@ -156,7 +209,7 @@ def register_builtins():
             return b(*args)
         return wrapped_builtin
 
-    for (name, builtin) in vars(__builtins__).iteritems():
+    for (name, builtin) in vars(__builtins__).items():
         if callable(builtin):
             if not name.startswith('_') and name == name.lower():
                 ns[name] = wrap_builtin(builtin)
@@ -167,7 +220,7 @@ def register_builtins():
             return b(*args)
         return wrapped_method
 
-    for (name, method) in vars(unicode).iteritems():
+    for (name, method) in vars(unicode).items():
         if callable(method):
             if not name.startswith('_'):
                 ns[name] = wrap_str_method(method)
@@ -201,20 +254,27 @@ def parse_options():
     parser.add_option("-q", "--quiet",
                       action="store_false", dest="verbose", default=True,
                       help="don't print status messages to stdout")
+    parser.add_option("-t", "--root-tag",
+                      dest="root_tag", metavar="TAG",
+                      help="surround output with <TAG>...</TAG> to produce a well-formed XML document")
     parser.add_option("-p", "--plain",
                       action="store_false", dest="pretty_print", default=True,
                       help="do not pretty-print the output")
-    parser.add_option("-N", "--ns",
-                      action="append", default=[],
-                      dest="namespaces",
-                      help="add a namespace declaration: --ns PREFIX=NS",)
-
+    parser.add_option("-l", "--lines",
+                      action="store_true", dest="line_by_line", default=False,
+                      help="parse each line of input separately (e.g. grep output)")
+    parser.add_option("-e", "--encoding",
+                      dest="encoding",
+                      help="use a specific encoding for parsing (may be required with --lines)")
+    parser.add_option("-N", "--ns", metavar="PREFIX=NS",
+                      action="append", dest="namespaces", default=[],
+                      help="add a namespace declaration")
 
     options, args = parser.parse_args()
 
     if options.long_help:
         parser.print_help()
-        print __doc__[__doc__.find('\n\n')+1:]
+        print(__doc__[__doc__.find('\n\n')+1:])
         sys.exit(0)
 
     if len(args) < 1:
@@ -236,18 +296,26 @@ def main(options, args):
         namespaces[prefix.strip()] = NS.strip()
 
     xpath = et.XPath(args[0], namespaces=namespaces)
+    files = args[1:] or [sys.stdin]
+
+    if options.root_tag and options.verbose:
+        print('<%s>' % options.root_tag)
 
     found = False
-    if len(args) == 1:
-        found = find_in_file(
-            sys.stdin, xpath, False, options.xinclude,
-            options.pretty_print)
-    else:
-        print_name = len(args) > 2
-        for filename in itertools.islice(args, 1, None):
-            found |= find_in_file(
-                filename, xpath, print_name, options.xinclude,
-                options.pretty_print)
+    print_name = len(files) > 1 and not options.root_tag
+    for input in files:
+        found |= find_in_file(
+            input, xpath,
+            print_name=print_name,
+            xinclude=options.xinclude,
+            pretty_print=options.pretty_print,
+            line_by_line=options.line_by_line,
+            encoding=options.encoding,
+            verbose=options.verbose,
+        )
+
+    if options.root_tag and options.verbose:
+        print('</%s>' % options.root_tag)
 
     return found
 
@@ -259,8 +327,8 @@ if __name__ == "__main__":
             sys.exit(0)
         else:
             sys.exit(1)
-    except et.XPathSyntaxError, e:
-        print >> sys.stderr, "Err: %s" % e
+    except et.XPathSyntaxError:
+        error(sys.exc_info()[1])
         sys.exit(4)
     except KeyboardInterrupt:
         pass
